#include <Wire.h>

//Global Variables
signed int deltaX=0, deltaY=0;
signed int deltaXB=0, deltaYB=0;
bool biteEnabled = false;

//Prototypes
void sensorInit();
void sensorGetPos();

void setup() 
{
    Wire.begin();
    sensorInit();
}

void loop() 
{
  int biteForce=0;
  byte num=0;
  byte error;
  
  //Check for click (bite)
  biteForce = analogRead(7);     // read the input pin
  //Serial.println(biteForce);     // debug value
 
  if(biteForce>550) //Below arbitrary activation force
  {
    if(biteEnabled == true)
    {
      biteEnabled = false;
      //Put sensor in power down mode
      Wire.beginTransmission(117);
      Wire.write(0x06);  // configuration register
      Wire.write(0xFF);  // address low byte (including PD_EnH bit)
      error = Wire.endTransmission();
      Serial.println("biteDisabled!");
    }
  }
   
  if(biteForce<=550) //Light pressure
  { 
    //Power up tracking if not already enabled
    if(biteEnabled == false)
    {
      biteEnabled = true;
      Wire.beginTransmission(117);
      Wire.write(0x06);  // configuration register
      Wire.write(0x17);  // POR value (not including PD_EnH bit)
      error = Wire.endTransmission();

      sensorInit(); //re-init the sensor (clear any old moves)
      
      Serial.println(biteForce);     // debug value
      Serial.println("biteEnabled!");
    }
    else //Already enabled, check 
    {
      sensorGetPos(); //Loads position delta into gloabal variables
      Mouse.move(deltaX, deltaY);    
    }
  }
  else //Motion flag is not set
  {
    //Serial.print(num, DEC);
    //Serial.println(" ! Motion_Status - No Motion");
  } 

  if(biteForce<=10) //force sensor is pressed
  {
    Serial.println("Click!");
    //Mouse.click();
    delay(50);     
  }   
  delay(8); 
}


//FUNCTIONS:

void sensorInit()
{
  byte num=0;
  byte error;
  
  // optical tracking sensor init 
  // check to see if the optical tracking sesnor is connected
  Wire.beginTransmission(117);
  Wire.write(0x00);  // address high byte
  error = Wire.endTransmission();
  
  // read 1 byte, from address 0
  Wire.requestFrom(117, 1);
  while(Wire.available())
  {
    num = Wire.read();
    if (num == 0x31)
    {
      Serial.println("Sensor Found!");
      
      //Initial configuration sequence 
      // switch to register bank0
      Wire.beginTransmission(117);
      Wire.write(0x7F);  // configuration register address
      Wire.write(0x00);  // sensor reset value
      error = Wire.endTransmission(); 
      
      // reset the OTS
      Wire.beginTransmission(117);
      Wire.write(0x06);  // configuration register address
      Wire.write(0x97);  // sensor reset value
      error = Wire.endTransmission();
    
      // wait 1ms for sensor to reset
      delay(1);
      
      // re-init the i2c connection for a fresh start
      Wire.begin();
    
      // ensure the sensor has left the reset state.
      Wire.beginTransmission(117);
      Wire.write(0x06);  // configuration register address
      Wire.write(0x17);  // normal run mode value
      error = Wire.endTransmission();
    
      // disable write protect
      Wire.beginTransmission(117);
      Wire.write(0x09);  // address high byte
      Wire.write(0x5A);  // address low byte
      error = Wire.endTransmission();
    
      // set X-axis resolution
      Wire.beginTransmission(117);
      Wire.write(0x0D);  // address high byte
      //Wire.write(0x65);  // set X-axis resolution
      Wire.write(33);  // set X-axis resolution
      error = Wire.endTransmission();
    
      // set Y-axis resolution
      Wire.beginTransmission(117);
      Wire.write(0x0E);  // address high byte
      //Wire.write(0xFF);  // set Y-axis resolution
      Wire.write(33);  // set Y-axis resolution
      error = Wire.endTransmission();

      
      // set 12 bit output mode
      Wire.beginTransmission(117);
      Wire.write(0x19);  // address high byte
      Wire.write(0x04);  // set 8 bit output sensor mode
      error = Wire.endTransmission();
      
      // enable write protection
      Wire.beginTransmission(117);
      Wire.write(0x09);  // address high byte
      Wire.write(0x00);  // write protect value
      error = Wire.endTransmission();  
    }
    else
    {
      Serial.println("Sensor not Found!");      
    }
  } 
}

void sensorGetPos()
{ 
  byte error;
  byte num=0;
  signed int deltaX_l=0, deltaY_l=0, deltaXY_h=0; 
  signed int deltaX_h=0, deltaY_h=0;

  //clear the global variables for position offset
  deltaX=0;
  deltaY=0;
  
  // Pass position information to host
  // read the motion register regardless of bite pressure
  Wire.beginTransmission(117);
  Wire.write(0x02);  // motion register
  error = Wire.endTransmission();
  // read 1 byte, from address 0
  Wire.requestFrom(117, 1);
  while(Wire.available())
  {
    num = Wire.read();  
    if(num & 0x80) //check motion bit in bit7
    {    
      Serial.println("motion detected!");
      //Read the motion registers
    
      // read the x data
      Wire.beginTransmission(117);
      Wire.write(0x03);  // Delta_X_Lo
      error = Wire.endTransmission();  
      // read 1 byte, from Delta_X_Lo
      Wire.requestFrom(117, 1);
      while(Wire.available())
      {
        deltaX_l = Wire.read();
      }
    
      // read the y data
      Wire.beginTransmission(117);
      Wire.write(0x04);  // Delta_Y_Lo
      error = Wire.endTransmission();   
      // read 1 byte, from Delta_Y_Lo
      Wire.requestFrom(117, 1);
      while(Wire.available())
      {
        deltaY_l = Wire.read();
      }

      // read the high nibble for X and Y motion (for 12 bit data)
      Wire.beginTransmission(117);
      Wire.write(0x12);  // Delta_XY_Hi
      error = Wire.endTransmission();  
      // read 1 byte, from Delta_XY_Hi
      Wire.requestFrom(117, 1);
      while(Wire.available())
      {
        deltaXY_h = Wire.read();
      }
    
      //Motion data aquired - convert to integers
      deltaX_h = (deltaXY_h<<4) & 0xF00;
      deltaX_h = (deltaX_h << 8) >> 8 ;
    
      if(deltaX_h & 0x800)    deltaX_h |= 0xfffff000; // 12-bit data convert to 16-bit 
    
      deltaX = (deltaX_h | deltaX_l); //Not inverted
      
      deltaY_h = (deltaXY_h<<8) & 0xF00;
      if(deltaY_h & 0x800)    deltaY_h |= 0xfffff000; // 12-bit data convert to 16-bit
      deltaY_h = (deltaY_h << 8) >> 8 ;
    
      deltaY = -(deltaY_h | deltaY_l); //Inverted
      
      Serial.print(deltaX);
      Serial.println(" : X");
      Serial.print(deltaY);
      Serial.println(" : Y");
    /*
      //scale the position output
      deltaX = (deltaX*1.7);
      deltaY = (deltaY*1.7);

      //set limits on move size
      if(deltaX>30)
      {
        deltaX = 20;
      }
      if(deltaY>20)
      {
        deltaY = 20;
      }
      if(deltaX<-20)
      {
        deltaY = -20;
      }
      if(deltaY<-20)
      {
        deltaY = -20;
      }
      */
    } 
  }
}

